@@ -59,11 +59,9 @@ int main(int argc, char* argv[])
 	status = freerdp_client_settings_parse_command_line(context->settings, argc, argv, FALSE);
 	if (status)
 	{
-		BOOL list = settings->ListMonitors;
-
 		rc = freerdp_client_settings_command_line_status_print(settings, status, argc, argv);
 
-		if (list)
+		if (settings->ListMonitors)
 			xf_list_monitors(xfc);
 
 		goto out;
