@@ -1,3 +1,4 @@
+#include <winpr/file.h>
 #include <freerdp/crypto/crypto.h>
 
 typedef char* (*get_field_pr)(X509*);
@@ -27,11 +28,7 @@ static char* certificate_path(void)
 	where the tests are run. (ie. no chdir occurs between compilation and test running, or __FILE__
 	is an absolute path).
 	*/
-#if defined(_WIN32)
-	static const char dirsep = '\\';
-#else
 	static const char dirsep = '/';
-#endif
 	static const char filename[] = "Test_x509_cert_info.pem";
 #ifdef TEST_SOURCE_DIR
 	const char* file = TEST_SOURCE_DIR;
@@ -91,7 +88,7 @@ static int TestCertificateFile(const char* certificate_path,
                                const certificate_test_t* certificate_tests, int count)
 {
 	X509* certificate;
-	FILE* certificate_file = fopen(certificate_path, "r");
+	FILE* certificate_file = winpr_fopen(certificate_path, "r");
 	int success = 0;
 	int i;
 
